#ifndef WINDOWMANAGER_H
#define WINDOWMANAGER_H

#include <QObject>
#include <QString>
#include <QIcon>
#include "windowinfo.h"
#include "wmregister.h"

namespace kdk
{
using WindowId = QVariant;
class WindowManager : public QObject
{
    Q_OBJECT

public:

    /**
     * @brief self
     * @return
     */
    static WindowManager* self();

    /**
      * @brief 获取窗口信息
      * @param windowId
      * @return
      */
    static WindowInfo getwindowInfo(const WindowId& windowId);

    /**
     * @brief 获取当前活动窗口
     * @return
     */
    static WindowId currentActiveWindow();

    /**
     * @brief 置顶窗口
     * @param windowId
     */
    static void keepWindowAbove(const WindowId& windowId);

    /**
     * @brief 获取窗口标题
     * @param windowId
     * @return
     */
     static QString getWindowTitle(const WindowId& windowId);

    /**
     * @brief 获取窗口图标
     * @param windowId
     * @return
     */
     static QIcon getWindowIcon(const WindowId& windowId);


     /**
      * @brief 获取窗口所在组的组名
      * @param windowId
      * @return
      */
     static QString getWindowGroup(const WindowId& windowId);


    /**
     * @brief 关闭窗口
     * @param windowId
     */
     static void closeWindow(const WindowId& windowId);

    /**
     * @brief 激活窗口
     * @param windowId
     */
     static void activateWindow(const WindowId& windowId);

    /**
     * @brief 最大化窗口
     * @param windowId
     */
     static void maximizeWindow(const WindowId& windowId);

    /**
     * @brief 最小化窗口
     * @param windowId
     */
     static void minimizeWindow(const WindowId& windowId);

     /**
      * @brief 获取窗口进程pid
      * @return
      */
     static quint32 getPid(const WindowId& windowId);

    /**
     * @brief 显示当前桌面
     */
     static void showDesktop();

    /**
     * @brief 取消显示当前桌面
     */
     static void hideDesktop();

     /**
     * @brief 获取当前桌面的名称
     * @return
     */
    static QString currentDesktop();

    /**
     * @brief 获取当前窗口列表
     * @return
     */
    static QList<WindowId> windows();

    /**
     * @brief 获取窗口类型，仅适用于X环境下，wayland下统一返回normal
     * @param windowId
     * @return
     */
    static NET::WindowType getWindowType(const WindowId& windowId);

    /**
     * @brief 设置窗口位置
     * @param window
     * @param rect
     */
    static void	setGeometry(QWindow *window,const QRect &rect);

Q_SIGNALS:
     /**
      * @brief 窗口添加信号
      * @param windowId
      */
     void windowAdded(const WindowId& windowId);

     /**
      * @brief 窗口删除信号
      * @param windowId
      */
     void windowRemoved(const WindowId& windowId);

     /**
      * @brief 活动窗口改变信号
      * @param wid
      */
     void activeWindowChanged(const WindowId& wid);

     /**
      * @brief 窗口改变信号
      * @param wid
      */
     void windowChanged(const WindowId& wid);

     /**
      * @brief 当前桌面改变信号
      * @param wid
      */
     void currentDesktopChanged();

private:
    WindowManager(QObject *parent = nullptr);
};
}


#endif // WINDOWMANAGER_H
